 %% Main function
 %{
This first function includes:
    (1) Experimental data loading
    (2) Minimizing function
%}
function BB_Fitting
%% Load experimental data
data = load('Lud_exp_data_FRT2.txt'); % Load external text file
d(:,:) = data; % Create vector for its use in the minimizing function
%% Minimizing function
q_CO2_max0 = -0.3; q_But_max0 = 0.03; q_Cap_max0 = 0.005;  % Set initial guess (mol_i/mol_x/h)
c_T_Ac0 = 150; c_T_But0 = 34; % Set initial guess (mol_i/m3)
format long
fminsearch(@(p) minSSE(p,d),[q_CO2_max0 q_But_max0 q_Cap_max0 c_T_Ac0 c_T_But0])
end
%% Second function - Sum of Square Errors function
%{
This second function includes:
    (1) Global parameters definition
    (2) Parameters to be fitted definition
    (3) ODE solver 
    (4) Error calculation
%}
function SSE = minSSE(p,d)
% Name global parameters
global K_CO2 K_Ac K_But K_NADH K_NH4 C_I_Ac C_I_But C_I_Cap Y_cat_CO2 Y_cat_Ac Y_cat_H Y_cat_H2O Y_cat_NH4 Y_cat_Xr Y_cat_NAD Y_cat_NADH ...
       Y_an_CO2 Y_an_Ac Y_an_But Y_an_Cap Y_an_H Y_an_H2O Y_an_NH4 Y_an_Xr Y_an_NAD Y_an_NADH ...
       H_CO2 P q_CO2_max q_max_But q_max_Cap ms_max Gibbs_diss_CO2 GR_Ac GR_But GR_Cap...
       k_e0 alp F R T T0 Eb Ec c_NH40 V_fc V_BC V_R c_T_Ac c_T_But ...
       Gf_NAD Gf_CO2 Gf_NH4 Gf_Ac Gf_But Gf_Cap Gf_Xr Gf_H2O Gf_H ...
       Hf_NAD Hf_CO2 Hf_NH4 Hf_Ac Hf_But Hf_Cap Hf_Xr Hf_H2O Hf_H ... 
% Parameters
K_CO2 = 3.8; K_Ac = 0.27; K_But = 0.07; K_NADH = 0.1; K_NH4 = 0.15; C_I_Ac = 800; C_I_But = 285; C_I_Cap = 170; % Kinetic parameters (mol_i/m3)
Y_cat_CO2 = -2; Y_cat_NH4 = 0; Y_cat_Xr = 0; Y_cat_Ac = 1; Y_cat_H2O = 2; Y_cat_H = -3; Y_cat_NAD = 4; Y_cat_NADH = -4; % Catabolic stoichiometry (mol_i/mol_Ac)
Y_an_CO2 = -1; Y_an_NH4 = -0.2; Y_an_Xr = 1; Y_an_H2O = 1.5; Y_an_H = -1.9; Y_an_NAD = 2.1; Y_an_NADH = -2.1; Y_an_Ac = 0; Y_an_But = 0; Y_an_Cap = 0; % Anabolic stoichiometry (mol_i/mol_X)
ms_max = 0.07625; % Estimated with a temperature dependent Arrhenius-type equation (mol_CO2/mol_x/h)
H_CO2 = 33.4; P = 1; % Gas-phase related parameters [y_in_CO2 = CO2 partial pressure], [H_CO2 = Henry coefficient] and [p = Pressure (atm)]
k_e0 = 0.03; alp = 0.5; F = 96485.34; R = 8.31; T = 305; T0 = 298; % Constants and values from literature
% k_e0 (1/s), alp (unitless), F (C), R (J/K/mol), T and T0 (K)
Eb = -1.2; Ec = -0.32; % [Eb = Cathode potential] and [Ec = NADH/NAD+ redox potential] in (V)
V_fc = 0.0255/1000; V_BC = (370-63.6-25.5)/1000000; V_R = 0.37/1000; % Volumes of different compartments [V_fc = Biocathdoe], [V_BC = Bubble column] and [V_R = Reactor] (m3)
Gf_NAD = -21.85; Gf_CO2 = -394.4; Gf_NH4 = -79.4; Gf_Ac = -369.4; Gf_But = -352.6; Gf_Cap = -336; Gf_Xr = -67; Gf_H2O = -237.2; Gf_H = 0; % Gibbs energy of formation (kJ/mol)
Hf_NAD = 0; Hf_CO2 = -394.1; Hf_NH4 = -133.3; Hf_Ac = -485.8; Hf_But = -535; Hf_Cap = -583.8; Hf_Xr = -91; Hf_H2O = -285.8; Hf_H = 0; % Heat of formation (kJ/mol)
GR_Ac = -55; GR_But = -198.2; GR_Cap = -341.6; % Gibbs energy of formation from CO2 (kJ/mol)
Gibbs_diss_CO2 = -1076; % Gibbs energy of CO2 dissipation (kJ/mol)
    % Set variables to be fitted and data points
q_CO2_max = p(1); q_max_But = p(2); q_max_Cap = p(3); c_T_Ac = p(4); c_T_But = p(5);
time = d(:,1);
SSE = 0;
%% ODE Solver
% set time interval (h) FTR1
tini=0;  tphase1 = 1104; tphase2 = 1560; tend = 3720;
    % Phase 1 (HRT = 4 d) (kla = 2.5 1/h) (Y_in_Co2 = 0.3)
D = 1/(4*24); % Dilution rate (1/h)
kla = 2.5;   % 1/h
y_in_CO2 = 0.3; % CO2 partial pressure
% set initial condition (values at t=0)
c_Xr0 = 5.2; % (mol_x/m3)
c_CO20 = P*y_in_CO2*H_CO2; c_Ac0 = 0.1/60*1000; c_But0 = 0.00000001/88*1000; c_Cap0 = 0.0000001/116*1000; c_NH40 = 3.74; c_NADH0 = 20; c_NAD0 = 0;  % (mol_i/mol3)
y0 = [c_Xr0; c_CO20; c_Ac0; c_But0; c_Cap0; c_NH40; c_NADH0; c_NAD0]; % Construct initial vector
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set tolerances and solve ODE system (t0= 0)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[~, y1]=ode15s(sys,tini:tphase1,y0,opt);
    % Phase 2 (HRT = 8 d) (kla = 2.5 1/h) (Y_in_Co2 = 0.3)
D = 1/(8*24); % Dilution rate (1/h)
kla = 2.5;   % 1/h
y_in_CO2 = 0.3; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 46 d)
y0 = [y1(end,1); y1(end,2); y1(end,3); y1(end,4); y1(end,5); y1(end,6); y1(end,7); y1(end,8)]; % Construct initial vector
% set tolerances and solve ODE system (t0= 46 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[~, y2]=ode15s(sys,tphase1:tphase2,y0,opt);
    % Phase 3 (HRT = 14 d) (kla = 2.5 1/h) (Y_in_Co2 = 0.3)
D = 1/(14*24); % Dilution rate (1/h)
kla = 2.5;   % 1/h
y_in_CO2 = 0.3; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 65 d)
y0 = [y2(end,1); y2(end,2); y2(end,3); y2(end,4); y2(end,5); y2(end,6); y2(end,7); y2(end,8)]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[~, y3]=ode15s(sys,tphase2:tend,y0,opt);
    % Concatenate results from all 4 phases
y = cat(1, y1(1:end-1,:), y2(1:end-1,:), y3);
%% Error calculation
    % Rename experimental data and fitted values 
c_Ac_fit = y(time,3); c_But_fit = y(time,4); c_Cap_fit = y(time,5); 
c_Ac_exp = d(:,2); c_But_exp = d(:,3); c_Cap_exp = d(:,4);
    % Calculate SSE
SSE_Ac = SSE + sum((c_Ac_exp - c_Ac_fit).^2);
SSE_But = SSE + sum((c_But_exp - c_But_fit).^2);
SSE_Cap = SSE + sum((c_Cap_exp - c_Cap_fit).^2);
SSE = SSE_Ac + SSE_But + SSE_Cap.^2;
end
%% Third function - System of equations 
%{
This third function includes:
    (1) Dynamic thermodynamic state analysis calculation
    (2) Kinetic rates definition
    (3) ODE system of equations (mass balances)
%}
function dy=dydt(~,y,D,kla,y_in_CO2) 
% Rename unknowns
c_Xr = y(1); c_CO2 = y(2); c_Ac = y(3); c_But = y(4); c_Cap = y(5); c_NH4 = y(6); c_NADH = y(7); c_NAD = y(8);
% Parameters - Global
global K_CO2 K_Ac K_But K_NADH C_I_Ac C_I_But C_I_Cap Y_cat_CO2 Y_cat_Ac Y_cat_H Y_cat_H2O Y_cat_NH4 Y_cat_NAD Y_cat_NADH ...
       Y_an_CO2 Y_an_Ac Y_an_H Y_an_H2O Y_an_NH4 Y_an_Xr Y_an_NAD Y_an_NADH ...
       H_CO2 P q_CO2_max ms_max Gibbs_diss_CO2 K_NH4 k_e0 alp F R T T0 Eb Ec c_NH40 V_fc V_BC V_R ...
       Gf_NAD Gf_CO2 Gf_NH4 Gf_Ac Gf_But Gf_Cap Gf_Xr Gf_H2O Gf_H GR_Ac GR_But GR_Cap ...
       Hf_NAD Hf_CO2 Hf_NH4 Hf_Ac Hf_But Hf_Cap Hf_Xr Hf_H2O Hf_H q_max_But q_max_Cap c_T_Ac c_T_But
% Parameters - Local
c_CO20 = P*y_in_CO2*H_CO2; % (mol/m3)
c_H = 10^(-5.8)*1000; c_H2O = 55.55; %(mol/L)
r_NAD = 20/1000; r_NADH = 20; % Mediator ratio for thermodynamic calculations
%% Inhibition terms
n_ac = 1; n_but = 1; n_cap = 1;
I_Ac = (1-c_Ac/C_I_Ac)^n_ac; I_But = (1-c_But/C_I_But)^n_but; I_Cap = (1-c_Cap/C_I_Cap)^n_cap; % (n) Product Inhibition 
%% Dynamic thermodynamic analysis - Stoichiometry determination
    % Standard energies calculation
Gibbs0_cat_R = Gf_NAD*Y_cat_NAD + Gf_CO2*Y_cat_CO2 + Gf_Ac*Y_cat_Ac + Gf_H2O*Y_cat_H2O + Gf_H*Y_cat_H; % Standard Gibbs energy of catabolic reaction
Gibbs0_an_R = Gf_NAD*Y_an_NAD + Gf_CO2*Y_an_CO2 + Gf_NH4*Y_an_NH4 + Gf_Xr*Y_an_Xr + Gf_H2O*Y_an_H2O + Gf_H*Y_an_H; % Standard Gibbs energy of anabolic reaction
Enthalpy0_cat_R = Hf_NAD*Y_cat_NAD + Hf_CO2*Y_cat_CO2 + Hf_Ac*Y_cat_Ac + Hf_H2O*Y_cat_H2O + Hf_H*Y_cat_H; % Standard Heat energy of catabolic reaction
Enthalpy0_an_R = Hf_NAD*Y_an_NAD + Hf_CO2*Y_an_CO2 + Hf_NH4*Y_an_NH4 + Hf_Xr*Y_an_Xr + Hf_H2O*Y_an_H2O + Hf_H*Y_an_H; % Standard Heat energy of anabolic reaction
% Account for Temperature
Gibbs0T_cat_R = Gibbs0_cat_R*T/T0 + Enthalpy0_cat_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of catabolic reaction
Gibbs0T_an_R = Gibbs0_an_R*T/T0 + Enthalpy0_an_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of anabolic reaction
% Account for concentrations (in mol/L)
Gibbs1T_cat_R = Gibbs0T_cat_R + R*T*(log(r_NAD/1000)*Y_cat_NAD + log(r_NADH/1000)*Y_cat_NADH + log(c_CO2/1000)*Y_cat_CO2 + log(c_Ac/1000)*Y_cat_Ac + log(c_H2O)*Y_cat_H2O + log(c_H/1000)*Y_cat_H)/1000; % Specific Gibbs energy of reaction of catabolic reaction
Gibbs1T_an_R = Gibbs0T_an_R + R*T*(log(r_NAD/1000)*Y_an_NAD + log(r_NADH/1000)*Y_an_NADH + log(c_CO2/1000)*Y_an_CO2 + log(c_NH4/1000)*Y_an_NH4 + log(c_Xr/1000)*Y_an_Xr ...
    + log(c_H2O)*Y_an_H2O + log(c_H/1000)*Y_an_H)/1000; % Specific Gibbs energy of reaction of anabolic reaction
    % Elongation Standard energies calculation
Gibbs0_But_R = Gf_NAD*2 + Gf_Ac*-2 + Gf_But*1 + Gf_H2O*2 + Gf_H*-3; % Standard Gibbs energy of butyrate elongation reaction
Gibbs0_Cap_R = Gf_NAD*2 + Gf_Ac*-1 + Gf_But*-1 + Gf_Cap*1 + Gf_H2O*2 + Gf_H*-3; % Standard Gibbs energy of caproate elongation reaction
Enthalpy0_But_R = Hf_NAD*2 + Hf_Ac*-2 + Hf_But*1 + Hf_H2O*2 + Hf_H*-3; % Standard Heat energy of butyrate elongation reaction
Enthalpy0_Cap_R = Hf_NAD*2 + Hf_Ac*-1 + Hf_But*-1 + Hf_Cap*1 + Hf_H2O*2 + Hf_H*-3; % Standard Heat energy of caproate elongation reaction
% Account for Temperature
Gibbs0T_But_R = Gibbs0_But_R*T/T0 + Enthalpy0_But_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of butyrate elongation reaction
Gibbs0T_Cap_R = Gibbs0_Cap_R*T/T0 + Enthalpy0_Cap_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of caproate elongation reaction
% Account for concentrations (in mol/L)
Gibbs1T_But_R = Gibbs0T_But_R + R*T*(log(r_NAD/1000)*2 + log(r_NADH/1000)*-2 + log(c_But/1000)*1 ...
    + log(c_Ac/1000)*-2 + log(c_H2O)*2 + log(c_H/1000)*-3)/1000; % Specific Gibbs energy of reaction of butyrate elongation reaction
Gibbs1T_Cap_R = Gibbs0T_Cap_R + R*T*(log(r_NAD/1000)*2 + log(r_NADH/1000)*-2 + log(c_Ac/1000)*-1 + log(c_But/1000)*-1 + log(c_Cap/1000)*1 ...
    + log(c_H2O)*2 + log(c_H/1000)*-3)/1000; % Specific Gibbs energy of reaction of caproate elongation reaction
% Thermodynamic product ratio (mol_i/mol_Ac)
Y_But_Ac = GR_Ac/GR_But; Y_Cap_Ac = GR_Ac/GR_Cap;
    % Calculate overall catabolic rate
lambda_cat = (Gibbs_diss_CO2 - Gibbs1T_an_R)/(Gibbs1T_cat_R - Y_But_Ac*Gibbs1T_But_R - Y_Cap_Ac*Gibbs1T_Cap_R);
    % Calculate growth stoichiometry (mol_i/mol_x)
Y_M_NADH = lambda_cat*Y_cat_NADH + Y_an_NADH; Y_M_NAD = lambda_cat*Y_cat_NAD + Y_an_NAD; Y_M_CO2 = lambda_cat*Y_cat_CO2 + Y_an_CO2; 
Y_M_NH4 = lambda_cat*Y_cat_NH4 + Y_an_NH4; Y_M_Ac = lambda_cat*Y_cat_Ac + Y_an_Ac;  
%% Specific rates - Growth, Consumption and Formation
    % Electron transfer 
k_f = k_e0*exp(-alp*F/R/T*(Eb - Ec)); % Forward Butler-Volmer term
k_r = k_e0*exp((1-alp)*F/R/T*(Eb - Ec)); % Reverse Butler-Volmer term
r_elec_m = k_f*c_NAD*c_H - k_r*c_NADH; % Electrochemical mediator rate (mol_mediator/m3/s)
    % Growth
q_CO2 = q_CO2_max*(c_CO2/(K_CO2 + c_CO2))*(c_NH4/(K_NH4 + c_NH4))*(c_NADH/(K_NADH + c_NADH))*I_Ac*I_But*I_Cap; % Specific CO2 uptake rate (mol_CO2/mol_x/h)
ms = ms_max; % Specific CO2 maintenance rate (mol_CO2/mol_x/h)
mu = (q_CO2 + ms)/Y_M_CO2; % Specific growth rate (1/h)
    % Specific Elongation production rates with Acetate & Butyrate threshold concentrations
% Acetate threshold
    A1 = 5/c_T_Ac; TC1 = 1.1*c_T_Ac; % Threshold constants
f1_Ac = 1/(1 + exp(A1*(c_T_Ac - c_Ac))); % Sigmoidal function to avoid negative rates when c_Ac < c_T_Ac
f2_Ac = 1/(1 + exp(A1*(TC1 - c_Ac))); % Sigmoidal function for smoothing the function at c_Ac close to c_T_Ac
% Butyrate threshold
    A2 = 10/c_T_But; TC2 = 1.1*c_T_But; % Threshold constants
f1_But = 1/(1 + exp(A2*(c_T_But - c_But))); % Sigmoidal function to avoid negative rates when c_Ac < c_T_Ac
f2_But = 1/(1 + exp(A2*(TC2 - c_But))); % Sigmoidal function for smoothing the function at c_Ac close to c_T_Ac
q_but = q_max_But*((c_Ac - c_T_Ac*f1_Ac)/(K_Ac + c_Ac - c_T_Ac*f1_Ac))*f2_Ac*I_Ac*I_But*I_Cap; % Specific Butyrate elongation rate (mol_But/mol_x/h)
q_Cap = q_max_Cap*((c_Ac - c_T_Ac*f1_Ac)/(K_Ac + c_Ac - c_T_Ac*f1_Ac))*f2_Ac*((c_But - c_T_But*f1_But)/(K_But + c_But - c_T_But*f1_But))*f2_But*I_Ac*I_But*I_Cap; % Specific Caproate elongation rate (mol_But/mol_x/h)
    % NAD/NADH
q_c_NADH = Y_M_NADH*mu - ms*(Y_cat_NADH/Y_cat_CO2) - 2*q_but - 2*q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_NADH/mol_x/h)
q_f_NAD = Y_M_NAD*mu - ms*(Y_cat_NAD/Y_cat_CO2) + 2*q_but + 2*q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_NAD/mol_x/h)
    % Acetate
q_t_Ac = Y_M_Ac*mu - ms*(Y_cat_Ac/Y_cat_CO2) - 2*q_but - q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_Ac/mol_x/h)
    % Butyrate
q_t_But = q_but - q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_But/mol_x/h)
    % Caproate
q_t_Cap = q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_Cap/mol_x/h)
    % NH4
q_t_NH4 = Y_M_NH4*mu - ms*(Y_cat_NH4/Y_cat_CO2); % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_NH4/mol_x/h)
%% Biological volumetric rates (mol_i/m3/h)
r_Xr = mu*c_Xr; % Cell rate 
r_CO2 = q_CO2*c_Xr; % CO2 rate
r_Ac = q_t_Ac*c_Xr; % Acetate rate
r_But = q_t_But*c_Xr; % Butyrate rate
r_Cap = q_t_Cap*c_Xr; % Caproate rate
r_NH4 = q_t_NH4*c_Xr; % Ammonium rate
r_bio_NADH = q_c_NADH*c_Xr; % NADH rate
r_bio_NAD = q_f_NAD*c_Xr; % NAD rate
%% Calculate derivatives
dy(1) = r_Xr; 
dy(2) = D*(0-c_CO2) + (V_fc/V_R)*r_CO2 + kla*(c_CO20-c_CO2)*(V_BC/V_R);
dy(3) = D*(0-c_Ac) + (V_fc/V_R)*r_Ac;
dy(4) = D*(0-c_But) +  (V_fc/V_R)*r_But;
dy(5) = D*(0-c_Cap) +  (V_fc/V_R)*r_Cap;
dy(6) = D*(c_NH40-c_NH4) +  (V_fc/V_R)*r_NH4;
dy(7) = (V_fc/V_R)*r_bio_NADH + (V_fc/V_R)*r_elec_m*3600;
dy(8) = (V_fc/V_R)*r_bio_NAD - (V_fc/V_R)*r_elec_m*3600;
dy=dy';
end