%% Main function (plot results from fitting - FTR2)
%{
This first function includes:
    (1) Global parameters definition
    (2) ODE solver 
    (3) Plotting of the results
    (4) Export results to Excel
%}
function BB_Marshall2013
close all
% Name global parameters
global K_CO2 K_Ac K_But K_NH4 K_NADH C_I_Ac C_I_But C_I_Cap Y_cat_CO2 Y_cat_Ac Y_cat_H Y_cat_H2O Y_cat_NH4 Y_cat_Xr Y_cat_NAD Y_cat_NADH ...
       Y_an_CO2 Y_an_Ac Y_an_But Y_an_Cap Y_an_H Y_an_H2O Y_an_NH4 Y_an_Xr Y_an_NAD Y_an_NADH ...
       H_CO2 p q_CO2_max q_max_But q_max_Cap ms_max Gibbs_diss_CO2 GR_Ac GR_But GR_Cap...
       k_e0 alp F R T T0 Eb Ec V_fc V_BC V_R c_T_Ac c_T_But pH ...
       Gf_NAD Gf_CO2 Gf_NH4 Gf_Ac Gf_But Gf_Cap Gf_Xr Gf_H2O Gf_H ...
       Hf_NAD Hf_CO2 Hf_NH4 Hf_Ac Hf_But Hf_Cap Hf_Xr Hf_H2O Hf_H
% Parameters
K_CO2 = 3.8;  K_Ac = 0.27; K_But = 0.07; K_NADH = 0.1; K_NH4 = 0.15; C_I_Ac = 800; C_I_But = 285; C_I_Cap = 170; % Kinetic parameters (mol_i/m3)
Y_cat_CO2 = -2; Y_cat_NH4 = 0; Y_cat_Xr = 0; Y_cat_Ac = 1; Y_cat_H2O = 2; Y_cat_H = -3; Y_cat_NAD = 4; Y_cat_NADH = -4; % Catabolic stoichiometry (mol_i/mol_Ac)
Y_an_CO2 = -1; Y_an_NH4 = -0.2; Y_an_Xr = 1; Y_an_H2O = 1.5; Y_an_H = -1.9; Y_an_NAD = 2.1; Y_an_NADH = -2.1; Y_an_Ac = 0; Y_an_But = 0; Y_an_Cap = 0; % Anabolic stoichiometry (mol_i/mol_X)
ms_max = 0.07625; % Estimated with a temperature dependent Arrhenius-type equation (mol_CO2/mol_x/h)
 H_CO2 = 33.4; p = 1; % Gas-phase related parameters [H_CO2 = Henry coefficient] and [p = Pressure (atm)]
k_e0 = 0.03; alp = 0.5; F = 96485.34; R = 8.31; T = 305; T0 = 298; % Constants and values from literature
% k_e0 (1/s), alp (unitless), F (C), R (J/K/mol), T and T0 (K)
Eb = -1.2; Ec = -0.32; % [Eb = Cathode potential] and [Ec = NADH/NAD+ redox potential] in (V)
% Volumes for Marshall et al. (2013) 
V_fc = 15/1000000; V_BC = (150-15)/1000000; V_R = 0.15/1000; % Volumes of different compartments [V_fc = Biocathdoe], [V_BC = Bubble column] and [V_R = Reactor] (m3)
Gf_NAD = -21.85; Gf_CO2 = -394.4; Gf_NH4 = -79.4; Gf_Ac = -369.4; Gf_But = -352.6; Gf_Cap = -336; Gf_Xr = -67; Gf_H2O = -237.2; Gf_H = 0; % Gibbs energy of formation (kJ/mol)
Hf_NAD = 0; Hf_CO2 = -394.1; Hf_NH4 = -133.3; Hf_Ac = -485.8; Hf_But = -535; Hf_Cap = -583.8; Hf_Xr = -91; Hf_H2O = -285.8; Hf_H = 0; % Heat of formation (kJ/mol)
GR_Ac = -55; GR_But = -198.2; GR_Cap = -341.6; % Gibbs energy of formation from CO2 (kJ/mol)
Gibbs_diss_CO2 = -1076; % Gibbs energy of CO2 dissipation (kJ/mol)
pH = 8;  %  pH for the validation 
% Rates obtained from the fitting script 'BB_SSE_Fitting' (mol_i/mol_x/h)
q_CO2_max = -0.307; q_max_But = 0.0212; q_max_Cap = 0.00464; 
% Threshold concentrations obtained from the fitting script 'BB_SSE_Fitting'(mol_i/m3)
c_T_Ac = 122.87; c_T_But = 43.04;

BB_sODE_Marshall
end

function BB_sODE_Marshall
% Name global parameters
global K_CO2 K_Ac K_But K_NH4 K_NADH C_I_Ac C_I_But C_I_Cap Y_cat_CO2 Y_cat_Ac Y_cat_H Y_cat_H2O Y_cat_NAD Y_cat_NADH ...
       Y_an_CO2 Y_an_Ac Y_an_H Y_an_H2O Y_an_NH4 Y_an_Xr Y_an_NAD Y_an_NADH ...
       H_CO2 p q_CO2_max q_max_But q_max_Cap ms_max Gibbs_diss_CO2 GR_Ac GR_But GR_Cap...
       k_e0 alp F R T T0 Eb Ec c_NH40 c_T_Ac c_T_But pH ...
       Gf_NAD Gf_CO2 Gf_NH4 Gf_Ac Gf_But Gf_Cap Gf_Xr Gf_H2O Gf_H ...
       Hf_NAD Hf_CO2 Hf_NH4 Hf_Ac Hf_But Hf_Cap Hf_Xr Hf_H2O Hf_H
%% ODE Solver
% set initial condition (values at t=0)
c_Xr0 = 1; % (mol_x/m3)
y_in_CO2 = 1; % CO2 partial pressure
c_CO20 = p*y_in_CO2*H_CO2; c_Ac0 = 0.1/60*1000; c_But0 = 0.00000001/88*1000; c_Cap0 = 0.0000001/116*1000; c_NH40 = 3.74; c_NADH0 = 20; c_NAD0 = 0;  % (mol_i/mol3)
tFBin = []; Xin = []; CO2in = []; Acin = []; Butin = []; Capin = []; NH4in = []; NADHin = []; NADin = [];
tFB = [tFBin,0]; X = [Xin,c_Xr0]; CO2 = [CO2in,c_CO20]; Ac = [Acin,c_Ac0]; But = [Butin,c_But0]; Cap = [Capin,c_Cap0]; NH4 = [NH4in,c_NH40]; NADH = [NADHin,c_NADH0]; NAD = [NADin,c_NAD0];
for n_FB = 0:1:1
% Time step
    tini = tFB(end,:); tday1 = tini + 0.33; tday1_sparging = tday1 + 23.67; 
                       tday2 = tday1_sparging + 0.33; tday2_sparging = tday2 + 23.67;
                       tday3 = tday2_sparging + 0.33; tday3_sparging = tday3 + 23.67;
                       tlast = tday3_sparging + 96;
    tp1 = tday1; tp2 = tday1_sparging;
    tp3 = tday2; tp4 = tday2_sparging;
    tp5 = tday3; tp6 = tday3_sparging;
    tend = tlast;
%% Day 1
    % Phase 1 
D = 0; % Dilution rate (1/h)
kla = 0.45;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% set initial condition (values at t=0)
X0 = X(end,:); CO20 = CO2(end,:); Ac0 = Ac(end,:); But0 = But(end,:); Cap0 = Cap(end,:); NH40 = NH4(end,:); NADH0 = NADH(end,:); NAD0 = NAD(end,:);
y0 = [X0; CO20; Ac0; But0; Cap0; NH40; NADH0; NAD0]; % Construct initial vector
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set tolerances and solve ODE system (t0= 0)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t1, y1]=ode15s(sys,tini:0.1:tp1,y0,opt);
    % Phase 2 
D = 0; % Dilution rate (1/h)
kla = 0;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 65 d)
y0 = [y1(end,1); y1(end,2); y1(end,3); y1(end,4); y1(end,5); y1(end,6); y1(end,7); y1(end,8)]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t2, y2]=ode15s(sys,tp1:0.1:tp2,y0,opt);
%% Day 2
    % Phase 3 
D = 0; % Dilution rate (1/h)
kla = 0.45;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% set initial condition (values at t= 65 d)
y0 = [y2(end,1); y2(end,2); y2(end,3); y2(end,4); y2(end,5); y2(end,6); y2(end,7); y2(end,8)]; % Construct initial vector
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set tolerances and solve ODE system (t0= 0)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t3, y3]=ode15s(sys,tp2:0.1:tp3,y0,opt);
    % Phase 4 
D = 0; % Dilution rate (1/h)
kla = 0;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 65 d)
y0 = [y3(end,1); y3(end,2); y3(end,3); y3(end,4); y3(end,5); y3(end,6); y3(end,7); y3(end,8)]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t4, y4]=ode15s(sys,tp3:0.1:tp4,y0,opt);
%% Day 3
    % Phase 5 
D = 0; % Dilution rate (1/h)
kla = 0.45;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% set initial condition (values at t= 65 d)
y0 = [y4(end,1); y4(end,2); y4(end,3); y4(end,4); y4(end,5); y4(end,6); y4(end,7); y4(end,8)]; % Construct initial vector
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set tolerances and solve ODE system (t0= 0)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t5, y5]=ode15s(sys,tp4:0.1:tp5,y0,opt);
    % Phase 6 
D = 0; % Dilution rate (1/h)
kla = 0;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 65 d)
y0 = [y5(end,1); y5(end,2); y5(end,3); y5(end,4); y5(end,5); y5(end,6); y5(end,7); y5(end,8)]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t6, y6]=ode15s(sys,tp5:0.1:tp6,y0,opt);
%% Day 4 - 7
    % Phase 7 
D = 0; % Dilution rate (1/h)
kla = 0.45;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 65 d)
y0 = [y6(end,1); y6(end,2); y6(end,3); y6(end,4); y6(end,5); y6(end,6); y6(end,7); y6(end,8)]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t7, y7]=ode15s(sys,tp6:0.1:tend,y0,opt);
    % Concatenate results from all 2 phases
y = cat(1, y1(1:end-1,:), y2(1:end-1,:), y3(1:end-1,:), y4(1:end-1,:), y5(1:end-1,:), y6(1:end-1,:), y7);
t = cat(1, t1(1:end-1), t2(1:end-1), t3(1:end-1), t4(1:end-1), t5(1:end-1), t6(1:end-1), t7);
tFB = [tFB;t]; X = [X;y(:,1)]; CO2 = [CO2;y(:,2)]; NADH = [NADH;y(:,7)]; NAD = [NAD;y(:,8)];
Ac = [Ac;y(:,3)]; But = [But;y(:,4)]; Cap = [Cap;y(:,5)]; NH4 = [NH4;y(:,6)]; 
end
    % Intermediate Phase 
tini = tFB(end,:); tInt = tini + 0.1; tend = tInt; 
D = 0; % Dilution rate (1/h)
kla = 0;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 133 d)
X0 = X(end,:); CO20 = CO2(end,:); Ac0 = 0.01*Ac(end,:); But0 = 0.001*But(end,:); Cap0 = 0.001*Cap(end,:); NH40 = NH4(end,:); NADH0 = NADH(end,:); NAD0 = NAD(end,:);
y0 = [X0; c_CO20; Ac0; But0; Cap0; c_NH40; NADH0; NAD0]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t, y]=ode15s(sys,tini:0.1:tend,y0,opt);
tFB = [tFB;t]; X = [X;y(:,1)]; CO2 = [CO2;y(:,2)]; Ac = [Ac;y(:,3)]; But = [But;y(:,4)]; Cap = [Cap;y(:,5)]; NH4 = [NH4;y(:,6)]; NADH = [NADH;y(:,7)]; NAD = [NAD;y(:,8)];  

for n_FB = 0:1:2
% Time step
    tini = tFB(end,:); tday1 = tini + 0.33; tday1_sparging = tday1 + 23.67; 
                       tday2 = tday1_sparging + 0.33; tday2_sparging = tday2 + 23.67;
                       tday3 = tday2_sparging + 0.33; tday3_sparging = tday3 + 23.67;
                       tlast = tday3_sparging + 96;
    tp1 = tday1; tp2 = tday1_sparging;
    tp3 = tday2; tp4 = tday2_sparging;
    tp5 = tday3; tp6 = tday3_sparging;
    tend = tlast;
%% Day 1
    % Phase 1 
D = 0; % Dilution rate (1/h)
kla = 0.45;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% set initial condition (values at t=0)
X0 = X(end,:); CO20 = CO2(end,:); Ac0 = Ac(end,:); But0 = But(end,:); Cap0 = Cap(end,:); NH40 = NH4(end,:); NADH0 = NADH(end,:); NAD0 = NAD(end,:);
y0 = [X0; CO20; Ac0; But0; Cap0; NH40; NADH0; NAD0]; % Construct initial vector
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set tolerances and solve ODE system (t0= 0)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t1, y1]=ode15s(sys,tini:0.1:tp1,y0,opt);
    % Phase 2 
D = 0; % Dilution rate (1/h)
kla = 0;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 65 d)
y0 = [y1(end,1); y1(end,2); y1(end,3); y1(end,4); y1(end,5); y1(end,6); y1(end,7); y1(end,8)]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t2, y2]=ode15s(sys,tp1:0.1:tp2,y0,opt);
%% Day 2
    % Phase 3 
D = 0; % Dilution rate (1/h)
kla = 0.45;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% set initial condition (values at t= 65 d)
y0 = [y2(end,1); y2(end,2); y2(end,3); y2(end,4); y2(end,5); y2(end,6); y2(end,7); y2(end,8)]; % Construct initial vector
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set tolerances and solve ODE system (t0= 0)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t3, y3]=ode15s(sys,tp2:0.1:tp3,y0,opt);
    % Phase 4 
D = 0; % Dilution rate (1/h)
kla = 0;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 65 d)
y0 = [y3(end,1); y3(end,2); y3(end,3); y3(end,4); y3(end,5); y3(end,6); y3(end,7); y3(end,8)]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t4, y4]=ode15s(sys,tp3:0.1:tp4,y0,opt);
%% Day 3
    % Phase 5 
D = 0; % Dilution rate (1/h)
kla = 0.45;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% set initial condition (values at t= 65 d)
y0 = [y4(end,1); y4(end,2); y4(end,3); y4(end,4); y4(end,5); y4(end,6); y4(end,7); y4(end,8)]; % Construct initial vector
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set tolerances and solve ODE system (t0= 0)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t5, y5]=ode15s(sys,tp4:0.1:tp5,y0,opt);
    % Phase 6 
D = 0; % Dilution rate (1/h)
kla = 0;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 65 d)
y0 = [y5(end,1); y5(end,2); y5(end,3); y5(end,4); y5(end,5); y5(end,6); y5(end,7); y5(end,8)]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t6, y6]=ode15s(sys,tp5:0.1:tp6,y0,opt);
%% Day 4 - 7
    % Phase 7 
D = 0; % Dilution rate (1/h)
kla = 0.45;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 65 d)
y0 = [y6(end,1); y6(end,2); y6(end,3); y6(end,4); y6(end,5); y6(end,6); y6(end,7); y6(end,8)]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t7, y7]=ode15s(sys,tp6:0.1:tend,y0,opt);
    % Concatenate results from all 2 phases
y = cat(1, y1(1:end-1,:), y2(1:end-1,:), y3(1:end-1,:), y4(1:end-1,:), y5(1:end-1,:), y6(1:end-1,:), y7);
t = cat(1, t1(1:end-1), t2(1:end-1), t3(1:end-1), t4(1:end-1), t5(1:end-1), t6(1:end-1), t7);
tFB = [tFB;t]; X = [X;y(:,1)]; CO2 = [CO2;y(:,2)]; Ac = [Ac;y(:,3)]; But = [But;y(:,4)]; Cap = [Cap;y(:,5)]; NH4 = [NH4;y(:,6)]; NADH = [NADH;y(:,7)]; NAD = [NAD;y(:,8)];
end

    % End Phase 
tini = tFB(end,:); tend = 1200; 
D = 0; % Dilution rate (1/h)
kla = 0.45;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 133 d)
X0 = X(end,:); CO20 = CO2(end,:); Ac0 = Ac(end,:); But0 = But(end,:); Cap0 = Cap(end,:); NH40 = NH4(end,:); NADH0 = NADH(end,:); NAD0 = NAD(end,:);
y0 = [X0; c_CO20; c_Ac0; c_But0; c_Cap0; c_NH40; NADH0; NAD0]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t, y]=ode15s(sys,tini:0.1:tend,y0,opt);
t = [tFB;t]; X = [X;y(:,1)]; CO2 = [CO2;y(:,2)]; Ac = [Ac;y(:,3)]; But = [But;y(:,4)]; Cap = [Cap;y(:,5)]; NH4 = [NH4;y(:,6)]; NADH = [NADH;y(:,7)]; NAD = [NAD;y(:,8)];  
 
%% Calculate rates for plotting
%{ 
From here on until the end of the first 'BB_sODE_Main' initial function, all calcualtions are done for plotting reasons. 
To see the actual ODE system code, please refer to the second funcion 'dy=dydt(~,y,D)'
%}
c_Xr = X; c_CO2 = CO2; c_Ac = Ac; c_But = But; c_Cap = Cap; c_NH4 = NH4; c_NADH = NADH; c_NAD = NAD; % Rename unknowns 
    % Electrochemical Current density
c_H = 10^(-pH)*1000; % (mol/m3)
k_f = k_e0*exp(-alp*F/R/T*(Eb - Ec)); % Forward Butler-Volmer term
k_r = k_e0*exp((1-alp)*F/R/T*(Eb - Ec)); % Reverse Butler-Volmer term
r_elec_m = k_f*c_NAD*c_H - k_r*c_NADH; % Electrochemical mediator rate (mol_mediator/m3/s)
I_v = r_elec_m*F*-2; % Electrochemical current density (A/m3)
% Parameters - Local
c_H2O = 55.55; % (mol/L)
r_NAD = 20/1000; r_NADH = 20; % Mediator ratio for thermodynamic calculations
%% Inhibition terms for plotting
n_ac = 1; n_but = 1; n_cap = 1;
I_Ac = (1-c_Ac/C_I_Ac).^n_ac; I_But = (1-c_But/C_I_But).^n_but; I_Cap = (1-c_Cap/C_I_Cap).^n_cap; % (n) Product Inhibition 
%% Dynamic thermodynamic analysis - Stoichiometry determination for plotting
    % Standard energies calculation
Gibbs0_cat_R = Gf_NAD*Y_cat_NAD + Gf_CO2*Y_cat_CO2 + Gf_Ac*Y_cat_Ac + Gf_H2O*Y_cat_H2O + Gf_H*Y_cat_H; % Standard Gibbs energy of catabolic reaction
Gibbs0_an_R = Gf_NAD*Y_an_NAD + Gf_CO2*Y_an_CO2 + Gf_NH4*Y_an_NH4 + Gf_Xr*Y_an_Xr + Gf_H2O*Y_an_H2O + Gf_H*Y_an_H; % Standard Gibbs energy of anabolic reaction
Enthalpy0_cat_R = Hf_NAD*Y_cat_NAD + Hf_CO2*Y_cat_CO2 + Hf_Ac*Y_cat_Ac + Hf_H2O*Y_cat_H2O + Hf_H*Y_cat_H; % Standard Heat energy of catabolic reaction
Enthalpy0_an_R = Hf_NAD*Y_an_NAD + Hf_CO2*Y_an_CO2 + Hf_NH4*Y_an_NH4 + Hf_Xr*Y_an_Xr + Hf_H2O*Y_an_H2O + Hf_H*Y_an_H; % Standard Heat energy of anabolic reaction
% Account for Temperature
Gibbs0T_cat_R = Gibbs0_cat_R*T/T0 + Enthalpy0_cat_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of catabolic reaction
Gibbs0T_an_R = Gibbs0_an_R*T/T0 + Enthalpy0_an_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of anabolic reaction
% Account for concentrations (in mol/L)
Gibbs1T_cat_R = Gibbs0T_cat_R + R*T*(log(r_NAD/1000)*Y_cat_NAD + log(r_NADH/1000)*Y_cat_NADH + log(c_CO2/1000)*Y_cat_CO2 + log(c_Ac/1000)*Y_cat_Ac + log(c_H2O)*Y_cat_H2O + log(c_H/1000)*Y_cat_H)/1000; % Specific Gibbs energy of reaction of catabolic reaction
Gibbs1T_an_R = Gibbs0T_an_R + R*T*(log(r_NAD/1000)*Y_an_NAD + log(r_NADH/1000)*Y_an_NADH + log(c_CO2/1000)*Y_an_CO2 + log(c_NH4/1000)*Y_an_NH4 + log(c_Xr/1000)*Y_an_Xr ...
    + log(c_H2O)*Y_an_H2O + log(c_H/1000)*Y_an_H)/1000; % Specific Gibbs energy of reaction of anabolic reaction
    % Elongation Standard energies calculation
Gibbs0_But_R = Gf_NAD*2 + Gf_Ac*-2 + Gf_But*1 + Gf_H2O*2 + Gf_H*-3; % Standard Gibbs energy of butyrate elongation reaction
Gibbs0_Cap_R = Gf_NAD*2 + Gf_Ac*-1 + Gf_But*-1 + Gf_Cap*1 + Gf_H2O*2 + Gf_H*-3; % Standard Gibbs energy of caproate elongation reaction
Enthalpy0_But_R = Hf_NAD*2 + Hf_Ac*-2 + Hf_But*1 + Hf_H2O*2 + Hf_H*-3; % Standard Heat energy of butyrate elongation reaction
Enthalpy0_Cap_R = Hf_NAD*2 + Hf_Ac*-1 + Hf_But*-1 + Hf_Cap*1 + Hf_H2O*2 + Hf_H*-3; % Standard Heat energy of caproate elongation reaction
% Account for Temperature
Gibbs0T_But_R = Gibbs0_But_R*T/T0 + Enthalpy0_But_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of butyrate elongation reaction
Gibbs0T_Cap_R = Gibbs0_Cap_R*T/T0 + Enthalpy0_Cap_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of caproate elongation reaction
% Account for concentrations (in mol/L)
Gibbs1T_But_R = Gibbs0T_But_R + R*T*(log(r_NAD/1000)*2 + log(r_NADH/1000)*-2 + log(c_But/1000)*1 ...
    + log(c_Ac/1000)*-2 + log(c_H2O)*2 + log(c_H/1000)*-3)/1000; % Specific Gibbs energy of reaction of butyrate elongation reaction
Gibbs1T_Cap_R = Gibbs0T_Cap_R + R*T*(log(r_NAD/1000)*2 + log(r_NADH/1000)*-2 + log(c_Ac/1000)*-1 + log(c_But/1000)*-1 + log(c_Cap/1000)*1 ...
    + log(c_H2O)*2 + log(c_H/1000)*-3)/1000; % Specific Gibbs energy of reaction of caproate elongation reaction
% Thermodynamic product ratio (mol_i/mol_Ac)
Y_But_Ac = GR_Ac/GR_But; Y_Cap_Ac = GR_Ac/GR_Cap;
    % Calculate overall catabolic rate
lambda_cat = (Gibbs_diss_CO2 - Gibbs1T_an_R)./(Gibbs1T_cat_R - Y_But_Ac*Gibbs1T_But_R - Y_Cap_Ac*Gibbs1T_Cap_R);
    % Calculate growth stoichiometry (mol_i/mol_x)
Y_M_NADH = lambda_cat*Y_cat_NADH + Y_an_NADH; Y_M_CO2 = lambda_cat*Y_cat_CO2 + Y_an_CO2; Y_M_Ac = lambda_cat*Y_cat_Ac + Y_an_Ac;  
%% Specific rates - Growth, Consumption and Formation for plotting
    % Growth
q_CO2 = q_CO2_max*(c_CO2./(K_CO2 + c_CO2)).*(c_NH4./(K_NH4 + c_NH4)).*(c_NADH./(K_NADH + c_NADH)).*I_Ac.*I_But.*I_Cap; % Specific CO2 uptake rate (mol_CO2/mol_x/h)
ms = ms_max; % Specific CO2 maintenance rate (mol_CO2/mol_x/h)
mu = (q_CO2 + ms)./Y_M_CO2; % Specific growth rate (1/h)
mu_max = (q_CO2_max + ms)./Y_M_CO2; % Maximum specific growth rate (1/h)
    % Specific Elongation production rates with Acetate & Butyrate threshold concentrations
% Acetate threshold
    A1 = 5/c_T_Ac; TC1 = 1.1*c_T_Ac; % Threshold constants
f1_Ac = 1./(1 + exp(A1.*(c_T_Ac - c_Ac))); % Sigmoidal function to avoid negative rates when c_Ac < c_T_Ac
f2_Ac = 1./(1 + exp(A1.*(TC1 - c_Ac))); % Sigmoidal function for smoothing the function at c_Ac close to c_T_Ac
% Butyrate threshold
    A2 = 10/c_T_But; TC2 = 1.1*c_T_But; % Threshold constants
f1_But = 1./(1 + exp(A2.*(c_T_But - c_But))); % Sigmoidal function to avoid negative rates when c_Ac < c_T_Ac
f2_But = 1./(1 + exp(A2.*(TC2 - c_But))); % Sigmoidal function for smoothing the function at c_Ac close to c_T_Ac
q_but = q_max_But*((c_Ac - c_T_Ac*f1_Ac)./(K_Ac + c_Ac - c_T_Ac*f1_Ac)).*f2_Ac.*I_Ac.*I_But.*I_Cap; % Specific Butyrate elongation rate (mol_But/mol_x/h)
q_Cap = q_max_Cap*((c_Ac - c_T_Ac*f1_Ac)./(K_Ac + c_Ac - c_T_Ac*f1_Ac)).*f2_Ac.*((c_But - c_T_But*f1_But)./(K_But + c_But - c_T_But*f1_But)).*f2_But.*I_Ac.*I_But.*I_Cap; % Specific Caproate elongation rate (mol_But/mol_x/h)
    % NADH
q_c_NADH = Y_M_NADH.*mu - ms*(Y_cat_NADH/Y_cat_CO2) - 2*q_but - 2*q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_NADH/mol_x/h)
    % Acetate
q_t_Ac = Y_M_Ac.*mu - ms*(Y_cat_Ac/Y_cat_CO2) - 2*q_but - q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_Ac/mol_x/h)
    % Butyrate
q_t_But = q_but - q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_But/mol_x/h)
    % Caproate
q_t_Cap = q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_Cap/mol_x/h)
% Biological Current density
I = 2*F*q_c_NADH.*X/3600; % (A/m3)
    % Monod Inhibition terms
I_Co2 = (c_CO2./(K_CO2 + c_CO2)); I_NH4 = (c_NH4./(K_NH4 + c_NH4)); I_NADH = (c_NADH./(K_NADH + c_NADH));
%% Biological volumetric rates (mol_i/m3/h)
r_Xr = mu.*c_Xr; % Cell rate 
r_Ac = q_t_Ac.*c_Xr; % Acetate rate
r_But = q_t_But.*c_Xr; % Butyrate rate
r_Cap = q_t_Cap.*c_Xr; % Caproate rate
%% Load experimental data
data = load('Marshall_exp_data.txt'); % Load external text file
    % Create data vectors
time = data(:,1); C_Ac = data(:,2);
    %% Plot results
% CO2 and NH4
figure('Name','Marshall et al. (2013)','NumberTitle','off'); set(gcf,'color','w');
subplot(3,2,2); xlabel('Time, t (d)'); axis([0 inf 0 35]);
yyaxis left; plot(t/24,CO2,'k','LineWidth',2); set(gca,'FontSize',10,'YColor','k');
 Phase1 = {'First Fed-Batch','    Operation'}; Phase2 = {'Second Fed-Batch','     Operation'}; Phase3 =  {'   Batch','Operation'};
ylabel('C_{CO_{2}} (mM)'); text(4,36,Phase1); text(20,36,Phase2); text(40,36,Phase3);
yyaxis right; plot(t/24,NH4,'b-','LineWidth',2); set(gca,'FontSize',10,'YColor','k'); 
ylabel('C_{NH_{4}^{+}} (mM)'); legend('CO_{2}','NH_{4}^{+}');
  xline(tini/24,'k--','LineWidth',2); xline(14,'k--','LineWidth',2);
% Current density
subplot(3,2,4); plot(t/24,I*15/1000,'g-',t/24,I_v*15/1000,'r-','LineWidth',2); set(gca,'FontSize',10); axis([0 inf -30 0]);
 xline(tini/24,'k--','LineWidth',2); xline(14,'k--','LineWidth',2); Phase1 = {'First Fed-Batch','    Operation'}; Phase2 = {'Second Fed-Batch','     Operation'}; Phase3 =  {'   Batch','Operation'};
 text(4,1,Phase1); text(20,1,Phase2); text(40,1,Phase3); 
xlabel('Time, t (d)'); ylabel('I (mA)'); legend('I^{Bio}','I^{Elec}');
% Product concentrations
subplot(3,2,1); plot(t/24,Ac,'b-',t/24,But,'r-',t/24,Cap,'k-',...
    time/24,C_Ac,'.b','LineWidth',2,'MarkerSize',15); set(gca,'FontSize',10); axis([0 inf 0 100]);
 xline(tini/24,'k--','LineWidth',2); xline(14,'k--','LineWidth',2); Phase1 = {'First Fed-Batch','    Operation'}; Phase2 = {'Second Fed-Batch','     Operation'}; Phase3 =  {'   Batch','Operation'};
 text(4,90,Phase1); text(20,90,Phase2); text(40,90,Phase3); 
xlabel('Time, t (d)'); ylabel('C_{i} (mM)'); legend('Acetate','Butyrate','Caproate','Acetate (Marshall,2013)','FontSize',8);
% Biomass
subplot(3,2,3); xlabel('Time, t (d)'); axis([0 inf 0 0.2]);
yyaxis left; plot(t/24,mu.*24,'g-',t/24,mu_max.*24,'m-','LineWidth',2); set(gca,'FontSize',10,'YColor','k');
 xline(tini/24,'k--','LineWidth',2); xline(14,'k--','LineWidth',2); Phase1 = {'First Fed-Batch','    Operation'}; Phase2 = {'Second Fed-Batch','     Operation'}; Phase3 =  {'   Batch','Operation'};
 text(4,0.14,Phase1); text(20,0.14,Phase2); text(40,0.14,Phase3); 
ylabel('mu (1 / d)');
yyaxis right; plot(t/24,X,'k-','LineWidth',2); set(gca,'FontSize',10,'YColor','k'); 
ylabel('C_{cells} (mM)'); legend('Growth rate','Max Growth rate','Cells');
% % Thermodynamic analysis
% subplot(3,2,5); xlabel('Time, t (d)'); axis([0 inf 0 inf]); title('Thermodynamic analysis');
% yyaxis left; plot(t/24,lambda_cat,'k-','LineWidth',2); set(gca,'FontSize',10,'YColor','k','box','off'); 
% ylabel('Catabolic rate (-)'); 
% yyaxis right; plot(t/24,Gibbs1T_cat_R,'b',t/24,Gibbs1T_an_R,'b',t/24,-Y_But_Ac*Gibbs1T_But_R,'r',t/24,-Y_Cap_Ac*Gibbs1T_Cap_R,'m',...
%     t/24,Gibbs1T_But_R,'-r',t/24,Gibbs1T_Cap_R,'--m','LineWidth',2); set(gca,'FontSize',10,'YColor','k','box','off'); hold on;
% ylabel('Free Gibbs Energy (kJ / mol_i)'); legend('Lambda_{Cat}','Gibbs^{1T}_{Cat}','Gibbs^{1T}_{An}','-Y^{But}_{Ac} Gibbs^{1T,Elg}_{But}','-Y^{Cap}_{Ac} Gibbs^{1T,Elg}_{Cap}','Gibbs^{1T,Elg}_{But}','Gibbs^{1T,Elg}_{Cap}');
% Product and Monod Inhibition terms
subplot(3,2,5); plot(t/24,I_Co2,'m-',t/24,I_NH4,'g-',t/24,I_Ac,'b-',t/24,I_But,'r-',t/24,I_Cap,'k-','LineWidth',2); set(gca,'FontSize',10); hold on;
 xline(tini/24,'k--','LineWidth',2); xline(14,'k--','LineWidth',2); Phase1 = {'First Fed-Batch','    Operation'}; Phase2 = {'Second Fed-Batch','     Operation'}; Phase3 =  {'   Batch','Operation'};
 text(4,1.02,Phase1); text(20,1.02,Phase2); text(40,1.02,Phase3); 
xlabel('Time, t (d)'); ylabel('Inhibtion term (-)');
legend('I^{M}_{CO_{2}}','I^{M}_{NH_{4}^{+}}','I^{p}_{Ac}','I^{p}_{But}','I^{p}_{Cap}');

%% Export results to excel (Uncomment to activate)
xlswrite('Results_Marshall2013_RawResults.xlsx',[t(:)/24,X(:),CO2(:),Ac(:),But(:),Cap(:),NH4(:),I(:)*15/1000]); % Exports raw data
xlswrite('Results_Marshall2013_CalculatedTerms.xlsx',[t(:)/24,mu(:)*24,mu_max(:)*24,I_Co2(:),I_NH4(:),I_Ac(:),I_But(:),I_Cap(:)]); % Exports inhibition terms and growth rates
end

%% Main function (plot results for validation - Marshall)
%{
This first function includes:
    (1) Global parameters definition
    (2) ODE solver 
    (3) Plotting of the results
    (4) Export results to Excel
%}
function BB_sODE_Joudin2018
% Name global parameters
global K_CO2 K_Ac K_But K_NH4 K_NADH C_I_Ac C_I_But C_I_Cap Y_cat_CO2 Y_cat_Ac Y_cat_H Y_cat_H2O Y_cat_NAD Y_cat_NADH ...
       Y_an_CO2 Y_an_Ac Y_an_H Y_an_H2O Y_an_NH4 Y_an_Xr Y_an_NAD Y_an_NADH ...
       H_CO2 p q_CO2_max q_max_But q_max_Cap ms_max Gibbs_diss_CO2 GR_Ac GR_But GR_Cap...
       k_e0 alp F R T T0 Eb Ec c_NH40 c_T_Ac c_T_But pH ...
       Gf_NAD Gf_CO2 Gf_NH4 Gf_Ac Gf_But Gf_Cap Gf_Xr Gf_H2O Gf_H ...
       Hf_NAD Hf_CO2 Hf_NH4 Hf_Ac Hf_But Hf_Cap Hf_Xr Hf_H2O Hf_H
%% ODE Solver
% set initial condition (values at t=0)
c_Xr0 = 2; % (mol_x/m3)
y_in_CO2 = 0.3; % CO2 partial pressure
c_CO20 = p*y_in_CO2*H_CO2; c_Ac0 = 0.1/60*1000; c_But0 = 0.00000001/88*1000; c_Cap0 = 0.0000001/116*1000; c_NH40 = 3.74; c_NADH0 = 20; c_NAD0 = 0;  % (mol_i/mol3)
tFBin = []; Xin = []; CO2in = []; Acin = []; Butin = []; Capin = []; NH4in = []; NADHin = []; NADin = [];
tFB = [tFBin,0]; X = [Xin,c_Xr0]; CO2 = [CO2in,c_CO20]; Ac = [Acin,c_Ac0]; But = [Butin,c_But0]; Cap = [Capin,c_Cap0]; NH4 = [NH4in,c_NH40]; NADH = [NADHin,c_NADH0]; NAD = [NADin,c_NAD0];
for n_FB = 0:1:33
% Time step
    tini = tFB(end,:); tphase1 = tini + 94; tphase2 = tphase1 + 1;
    tsparging = tphase1; tend = tphase2;
    % Phase 1 
D = 0; % Dilution rate (1/h)
kla = 0;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% set initial condition (values at t=0)
X0 = X(end,:); CO20 = CO2(end,:); Ac0 = Ac(end,:); But0 = But(end,:); Cap0 = Cap(end,:); NH40 = NH4(end,:); NADH0 = NADH(end,:); NAD0 = NAD(end,:);
y0 = [X0; CO20; Ac0; But0; Cap0; NH40; NADH0; NAD0]; % Construct initial vector
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set tolerances and solve ODE system (t0= 0)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t1, y1]=ode15s(sys,tini:tsparging,y0,opt);
    % Phase 2 
D = 0; % Dilution rate (1/h)
kla = 0.45;   % 1/h
y_in_CO2 = 1; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 65 d)
y0 = [y1(end,1); y1(end,2); y1(end,3); y1(end,4); y1(end,5); y1(end,6); y1(end,7); y1(end,8)]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t2, y2]=ode15s(sys,tsparging:tend,y0,opt);
    % Concatenate results from all 2 phases
y = cat(1, y1(1:end-1,:), y2);
t = cat(1, t1(1:end-1), t2);
tFB = [tFB;t]; X = [X;y(:,1)]; CO2 = [CO2;y(:,2)]; Ac = [Ac;y(:,3)]; But = [But;y(:,4)]; Cap = [Cap;y(:,5)]; NH4 = [NH4;y(:,6)]; NADH = [NADH;y(:,7)]; NAD = [NAD;y(:,8)];
end
    % Phase 3 (HRT = 4 d) (kla = 0.45 1/h) (Y_in_Co2 = 0.3)
tini = tFB(end,:); tphase1 = 4368; tend = 8600;
D = 1/(4*24); % Dilution rate (1/h)
kla = 0.45;   % 1/h
y_in_CO2 = 0.3; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t= 133 d)
X0 = X(end,:); CO20 = CO2(end,:); Ac0 = Ac(end,:); But0 = But(end,:); Cap0 = Cap(end,:); NH40 = NH4(end,:); NADH0 = NADH(end,:); NAD0 = NAD(end,:);
y0 = [X0; CO20; Ac0; But0; Cap0; NH40; NADH0; NAD0]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t1, y1]=ode15s(sys,tini:tphase1,y0,opt);
    % Phase 4 (HRT = 4 d) (kla = 0.45 1/h) (Y_in_Co2 = 0.3)
D = 1/(4*24); % Dilution rate (1/h)
kla = 0.45;   % 1/h
y_in_CO2 = 0.3; % CO2 partial pressure
% name function of ODE system
sys = @(t,y)dydt(t,y,D,kla,y_in_CO2);
% set initial condition (values at t=  d)
y0 = [y1(end,1); y1(end,2); y1(end,3); y1(end,4); y1(end,5); y1(end,6); y1(end,7); y1(end,8)]; % Construct initial vector
% set tolerances and solve ODE system (t0= 65 d)
opt = odeset('RelTol',1e-10, 'AbsTol',1e-10);
[t2, y2]=ode15s(sys,tphase1:tend,y0,opt);
    % Concatenate results from all 2 phases
y = cat(1, y1(1:end-1,:), y2);
t = cat(1, t1(1:end-1), t2);
t = [tFB;t]; X = [X;y(:,1)]; CO2 = [CO2;y(:,2)]; Ac = [Ac;y(:,3)]; But = [But;y(:,4)]; Cap = [Cap;y(:,5)]; NH4 = [NH4;y(:,6)]; NADH = [NADH;y(:,7)]; NAD = [NAD;y(:,8)];

%% Calculate rates for plotting
%{ 
From here on until the end of the first 'BB_sODE_Main' initial function, all calcualtions are done for plotting reasons. 
To see the actual ODE system code, please refer to the second funcion 'dy=dydt(~,y,D)'
%}
c_Xr = X; c_CO2 = CO2; c_Ac = Ac; c_But = But; c_Cap = Cap; c_NH4 = NH4; c_NADH = NADH; c_NAD = NAD; % Rename unknowns 
    % Electrochemical Current density
c_H = 10^(-pH)*1000; % (mol/m3)
k_f = k_e0*exp(-alp*2*F/R/T*(Eb - Ec)); % Forward Butler-Volmer term
k_r = k_e0*exp((1-alp)*2*F/R/T*(Eb - Ec)); % Reverse Butler-Volmer term
r_elec_m = k_f*c_NAD*c_H - k_r*c_NADH; % Electrochemical mediator rate (mol_mediator/m3/s)
I_v = r_elec_m*F*-2; % Electrochemical current density (A/m3)
% Parameters - Local
c_H2O = 55.55; % (mol/L)
r_NAD = 20/1000; r_NADH = 20; % Mediator ratio for thermodynamic calculations
%% Inhibition terms for plotting
n_ac = 1; n_but = 1; n_cap = 1;
I_Ac = (1-c_Ac/C_I_Ac).^n_ac; I_But = (1-c_But/C_I_But).^n_but; I_Cap = (1-c_Cap/C_I_Cap).^n_cap; % (n) Product Inhibition 
%% Dynamic thermodynamic analysis - Stoichiometry determination for plotting
    % Standard energies calculation
Gibbs0_cat_R = Gf_NAD*Y_cat_NAD + Gf_CO2*Y_cat_CO2 + Gf_Ac*Y_cat_Ac + Gf_H2O*Y_cat_H2O + Gf_H*Y_cat_H; % Standard Gibbs energy of catabolic reaction
Gibbs0_an_R = Gf_NAD*Y_an_NAD + Gf_CO2*Y_an_CO2 + Gf_NH4*Y_an_NH4 + Gf_Xr*Y_an_Xr + Gf_H2O*Y_an_H2O + Gf_H*Y_an_H; % Standard Gibbs energy of anabolic reaction
Enthalpy0_cat_R = Hf_NAD*Y_cat_NAD + Hf_CO2*Y_cat_CO2 + Hf_Ac*Y_cat_Ac + Hf_H2O*Y_cat_H2O + Hf_H*Y_cat_H; % Standard Heat energy of catabolic reaction
Enthalpy0_an_R = Hf_NAD*Y_an_NAD + Hf_CO2*Y_an_CO2 + Hf_NH4*Y_an_NH4 + Hf_Xr*Y_an_Xr + Hf_H2O*Y_an_H2O + Hf_H*Y_an_H; % Standard Heat energy of anabolic reaction
% Account for Temperature
Gibbs0T_cat_R = Gibbs0_cat_R*T/T0 + Enthalpy0_cat_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of catabolic reaction
Gibbs0T_an_R = Gibbs0_an_R*T/T0 + Enthalpy0_an_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of anabolic reaction
% Account for concentrations (in mol/L)
Gibbs1T_cat_R = Gibbs0T_cat_R + R*T*(log(r_NAD/1000)*Y_cat_NAD + log(r_NADH/1000)*Y_cat_NADH + log(c_CO2/1000)*Y_cat_CO2 + log(c_Ac/1000)*Y_cat_Ac + log(c_H2O)*Y_cat_H2O + log(c_H/1000)*Y_cat_H)/1000; % Specific Gibbs energy of reaction of catabolic reaction
Gibbs1T_an_R = Gibbs0T_an_R + R*T*(log(r_NAD/1000)*Y_an_NAD + log(r_NADH/1000)*Y_an_NADH + log(c_CO2/1000)*Y_an_CO2 + log(c_NH4/1000)*Y_an_NH4 + log(c_Xr/1000)*Y_an_Xr ...
    + log(c_H2O)*Y_an_H2O + log(c_H/1000)*Y_an_H)/1000; % Specific Gibbs energy of reaction of anabolic reaction
    % Elongation Standard energies calculation
Gibbs0_But_R = Gf_NAD*2 + Gf_Ac*-2 + Gf_But*1 + Gf_H2O*2 + Gf_H*-3; % Standard Gibbs energy of butyrate elongation reaction
Gibbs0_Cap_R = Gf_NAD*2 + Gf_Ac*-1 + Gf_But*-1 + Gf_Cap*1 + Gf_H2O*2 + Gf_H*-3; % Standard Gibbs energy of caproate elongation reaction
Enthalpy0_But_R = Hf_NAD*2 + Hf_Ac*-2 + Hf_But*1 + Hf_H2O*2 + Hf_H*-3; % Standard Heat energy of butyrate elongation reaction
Enthalpy0_Cap_R = Hf_NAD*2 + Hf_Ac*-1 + Hf_But*-1 + Hf_Cap*1 + Hf_H2O*2 + Hf_H*-3; % Standard Heat energy of caproate elongation reaction
% Account for Temperature
Gibbs0T_But_R = Gibbs0_But_R*T/T0 + Enthalpy0_But_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of butyrate elongation reaction
Gibbs0T_Cap_R = Gibbs0_Cap_R*T/T0 + Enthalpy0_Cap_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of caproate elongation reaction
% Account for concentrations (in mol/L)
Gibbs1T_But_R = Gibbs0T_But_R + R*T*(log(r_NAD/1000)*2 + log(r_NADH/1000)*-2 + log(c_But/1000)*1 ...
    + log(c_Ac/1000)*-2 + log(c_H2O)*2 + log(c_H/1000)*-3)/1000; % Specific Gibbs energy of reaction of butyrate elongation reaction
Gibbs1T_Cap_R = Gibbs0T_Cap_R + R*T*(log(r_NAD/1000)*2 + log(r_NADH/1000)*-2 + log(c_Ac/1000)*-1 + log(c_But/1000)*-1 + log(c_Cap/1000)*1 ...
    + log(c_H2O)*2 + log(c_H/1000)*-3)/1000; % Specific Gibbs energy of reaction of caproate elongation reaction
% Thermodynamic product ratio (mol_i/mol_Ac)
Y_But_Ac = GR_Ac/GR_But; Y_Cap_Ac = GR_Ac/GR_Cap;
    % Calculate overall catabolic rate
lambda_cat = (Gibbs_diss_CO2 - Gibbs1T_an_R)./(Gibbs1T_cat_R - Y_But_Ac*Gibbs1T_But_R - Y_Cap_Ac*Gibbs1T_Cap_R);
    % Calculate growth stoichiometry (mol_i/mol_x)
Y_M_NADH = lambda_cat*Y_cat_NADH + Y_an_NADH; Y_M_CO2 = lambda_cat*Y_cat_CO2 + Y_an_CO2; Y_M_Ac = lambda_cat*Y_cat_Ac + Y_an_Ac;  
%% Specific rates - Growth, Consumption and Formation for plotting
    % Growth
q_CO2 = q_CO2_max*(c_CO2./(K_CO2 + c_CO2)).*(c_NH4./(K_NH4 + c_NH4)).*(c_NADH./(K_NADH + c_NADH)).*I_Ac.*I_But.*I_Cap; % Specific CO2 uptake rate (mol_CO2/mol_x/h)
ms = ms_max; % Specific CO2 maintenance rate (mol_CO2/mol_x/h)
mu = (q_CO2 + ms)./Y_M_CO2; % Specific growth rate (1/h)
mu_max = (q_CO2_max + ms)./Y_M_CO2; % Maximum specific growth rate (1/h)
    % Specific Elongation production rates with Acetate & Butyrate threshold concentrations
% Acetate threshold
    A1 = 5/c_T_Ac; TC1 = 1.1*c_T_Ac; % Threshold constants
f1_Ac = 1./(1 + exp(A1.*(c_T_Ac - c_Ac))); % Sigmoidal function to avoid negative rates when c_Ac < c_T_Ac
f2_Ac = 1./(1 + exp(A1.*(TC1 - c_Ac))); % Sigmoidal function for smoothing the function at c_Ac close to c_T_Ac
% Butyrate threshold
    A2 = 10/c_T_But; TC2 = 1.1*c_T_But; % Threshold constants
f1_But = 1./(1 + exp(A2.*(c_T_But - c_But))); % Sigmoidal function to avoid negative rates when c_Ac < c_T_Ac
f2_But = 1./(1 + exp(A2.*(TC2 - c_But))); % Sigmoidal function for smoothing the function at c_Ac close to c_T_Ac
q_but = q_max_But*((c_Ac - c_T_Ac*f1_Ac)./(K_Ac + c_Ac - c_T_Ac*f1_Ac)).*f2_Ac.*I_Ac.*I_But.*I_Cap; % Specific Butyrate elongation rate (mol_But/mol_x/h)
q_Cap = q_max_Cap*((c_Ac - c_T_Ac*f1_Ac)./(K_Ac + c_Ac - c_T_Ac*f1_Ac)).*f2_Ac.*((c_But - c_T_But*f1_But)./(K_But + c_But - c_T_But*f1_But)).*f2_But.*I_Ac.*I_But.*I_Cap; % Specific Caproate elongation rate (mol_But/mol_x/h)
    % NADH
q_c_NADH = Y_M_NADH.*mu - ms*(Y_cat_NADH/Y_cat_CO2) - 2*q_but - 2*q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_NADH/mol_x/h)
    % Acetate
q_t_Ac = Y_M_Ac.*mu - ms*(Y_cat_Ac/Y_cat_CO2) - 2*q_but - q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_Ac/mol_x/h)
    % Butyrate
q_t_But = q_but - q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_But/mol_x/h)
    % Caproate
q_t_Cap = q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_Cap/mol_x/h)
% Biological Current density
I = 2*F*q_c_NADH.*X/3600; % (A/m3)
    % Monod Inhibition terms
I_Co2 = (c_CO2./(K_CO2 + c_CO2)); I_NH4 = (c_NH4./(K_NH4 + c_NH4)); I_NADH = (c_NADH./(K_NADH + c_NADH));
%% Biological volumetric rates (mol_i/m3/h)
r_Xr = mu.*c_Xr; % Cell rate 
r_Ac = q_t_Ac.*c_Xr; % Acetate rate
r_But = q_t_But.*c_Xr; % Butyrate rate
r_Cap = q_t_Cap.*c_Xr; % Caproate rate
%% Load experimental data
data = load('Lud_exp_data_2018.txt'); % Load external text file
    % Create data vectors
time = data(:,1); C_Ac = data(:,2); C_But = data(:,3); C_Cap = data(:,4);
    %% Plot results
% CO2 and NH4
figure('Name','Jourdin et al. (2018)','NumberTitle','off');
subplot(3,2,2); xlabel('Time, t (d)'); axis([0 inf 0 inf]);
yyaxis left; plot(t/24,CO2,'k','LineWidth',2); set(gca,'FontSize',10,'YColor','k','box','off'); 
xline(3230/24,'k--','LineWidth',2); Phase1 = {'Fed-Batch Operation','   '}; Phase2 = {'Continuous Operation','       (HRT = 4 d)'};
text(25,11.5,Phase1); text(200,11.5,Phase2); 
ylabel('C_{CO_{2}} (mM)'); 
yyaxis right; plot(t/24,NH4,'b','LineWidth',2); set(gca,'FontSize',10,'YColor','k','box','off');
ylabel('C_{NH_{4}^{+}} (mM)'); legend('CO_{2}','NH_{4}^{+}');
% Current density
subplot(3,2,4); plot(t/24,I/1000,'g-',t/24,I_v/1000,'r-','LineWidth',2); set(gca,'FontSize',10,'box','off');
xline(3230/24,'k--','LineWidth',2); Phase1 = {'Fed-Batch Operation','   '}; Phase2 = {'Continuous Operation','       (HRT = 4 d)'};
text(25,0.2,Phase1); text(200,0.2,Phase2); 
xlabel('Time, t (d)'); ylabel('I (kA / m^{3}_{e})'); legend('I^{Bio}','I^{Elec}');
% Product concentrations
subplot(3,2,1); plot(t/24,Ac,'b-',t/24,But,'r-',t/24,Cap,'k-',...
    time/24,C_Ac/60,'.b',time/24,C_But/88,'.r',time/24,C_Cap/116,'.k','LineWidth',2,'MarkerSize',15); set(gca,'FontSize',10,'box','off'); axis([0 inf 0 175]);
xline(3230/24,'k--','LineWidth',2); Phase1 = {'Fed-Batch Operation','   '}; Phase2 = {'Continuous Operation','       (HRT = 4 d)'};
text(25,175,Phase1); text(200,175,Phase2); 
xlabel('Time, t (d)'); ylabel('C_{i} (mM)'); legend('Acetate','Butyrate','Caproate','Acetate (Jourdin,2018)','Butyrate (Jourdin,2018)','Caproate (Jourdin,2018)','FontSize',8);
% Biomass
subplot(3,2,3); xlabel('Time, t (d)'); axis([0 inf 0 inf]);
yyaxis left; plot(t/24,mu.*24,'g-',t/24,mu_max.*24,'m-','LineWidth',2); set(gca,'FontSize',10,'YColor','k','box','off');
xline(3230/24,'k--','LineWidth',2); Phase1 = {'Fed-Batch Operation','   '}; Phase2 = {'Continuous Operation','       (HRT = 4 d)'};
text(25,0.16,Phase1); text(200,0.16,Phase2); 
ylabel('mu (1 / d)');
yyaxis right; plot(t/24,X,'k-','LineWidth',2); set(gca,'FontSize',10,'YColor','k','box','off'); 
ylabel('C_{cells} (mM)'); legend('Growth rate','Max Growth rate','Cells');
% % Thermodynamic analysis
% subplot(3,2,5); xlabel('Time, t (d)'); axis([0 inf 0 inf]); title('Thermodynamic analysis');
% yyaxis left; plot(t/24,lambda_cat,'k-','LineWidth',2); set(gca,'FontSize',10,'YColor','k','box','off'); 
% ylabel('Catabolic rate (-)'); 
% yyaxis right; plot(t/24,Gibbs1T_cat_R,'b',t/24,Gibbs1T_an_R,'b',t/24,-Y_But_Ac*Gibbs1T_But_R,'r',t/24,-Y_Cap_Ac*Gibbs1T_Cap_R,'m',...
%     t/24,Gibbs1T_But_R,'-r',t/24,Gibbs1T_Cap_R,'--m','LineWidth',2); set(gca,'FontSize',10,'YColor','k','box','off'); hold on;
% ylabel('Free Gibbs Energy (kJ / mol_i)'); legend('Lambda_{Cat}','Gibbs^{1T}_{Cat}','Gibbs^{1T}_{An}','-Y^{But}_{Ac} Gibbs^{1T,Elg}_{But}','-Y^{Cap}_{Ac} Gibbs^{1T,Elg}_{Cap}','Gibbs^{1T,Elg}_{But}','Gibbs^{1T,Elg}_{Cap}');
% Product and Monod Inhibition terms
subplot(3,2,5); plot(t/24,I_Co2,'m-',t/24,I_NH4,'g-',t/24,I_Ac,'b-',t/24,I_But,'r-',t/24,I_Cap,'k-','LineWidth',2); set(gca,'FontSize',10,'box','off'); hold on;
xlabel('Time, t (d)'); ylabel('Inhibtion term (-)');
xline(3230/24,'k--','LineWidth',2); Phase1 = 'Discontinuous Operation'; Phase2 = {'Continuous Operation','       (HRT = 4 d)'};
text(25,1.1,Phase1); text(200,1.1,Phase2); 
legend('I^{M}_{CO_{2}}','I^{M}_{NH_{4}^{+}}','I^{p}_{Ac}','I^{p}_{But}','I^{p}_{Cap}');

%% Export results to excel (Uncomment to activate)
% Ac=y(:,3); But=y(:,4); Cap=y(:,5); 
% xlswrite('ResultsKinetics_Marshall.xlsx',[Ac(:),But(:),Cap(:)]); % Exports product concentration data
end
%% Second function - System of equations
%{
This second function includes:
    (1) Dynamic thermodynamic state analysis calculation
    (2) Kinetic rates definition
    (3) ODE system of equations (mass balances)
%}
function dy=dydt(~,y,D,kla,y_in_CO2) 
% Rename unknowns
c_Xr = y(1); c_CO2 = y(2); c_Ac = y(3); c_But = y(4); c_Cap = y(5); c_NH4 = y(6); c_NADH = y(7); c_NAD = y(8);
% Parameters - Global
global K_CO2 K_Ac K_But K_NADH C_I_Ac C_I_But C_I_Cap Y_cat_CO2 Y_cat_Ac Y_cat_H Y_cat_H2O Y_cat_NH4 Y_cat_NAD Y_cat_NADH ...
       Y_an_CO2 Y_an_Ac Y_an_H Y_an_H2O Y_an_NH4 Y_an_Xr Y_an_NAD Y_an_NADH pH ...
       H_CO2 p q_CO2_max ms_max Gibbs_diss_CO2 K_NH4 k_e0 alp F R T T0 Eb Ec c_NH40 V_fc V_BC V_R ...
       Gf_NAD Gf_CO2 Gf_NH4 Gf_Ac Gf_But Gf_Cap Gf_Xr Gf_H2O Gf_H GR_Ac GR_But GR_Cap ...
       Hf_NAD Hf_CO2 Hf_NH4 Hf_Ac Hf_But Hf_Cap Hf_Xr Hf_H2O Hf_H q_max_But q_max_Cap c_T_Ac c_T_But
% Parameters - Local
c_CO20 = p*y_in_CO2*H_CO2; % (mol/m3)
c_H = 10^(-pH)*1000; c_H2O = 55.55; %(mol/L)
r_NAD = 20/1000; r_NADH = 20; % Mediator ratio for thermodynamic calculations
%% Inhibition terms
n_ac = 1; n_but = 1; n_cap = 1;
I_Ac = (1-c_Ac/C_I_Ac)^n_ac; I_But = (1-c_But/C_I_But)^n_but; I_Cap = (1-c_Cap/C_I_Cap)^n_cap; % (n) Product Inhibition 
%% Dynamic thermodynamic analysis - Stoichiometry determination
    % Standard energies calculation
Gibbs0_cat_R = Gf_NAD*Y_cat_NAD + Gf_CO2*Y_cat_CO2 + Gf_Ac*Y_cat_Ac + Gf_H2O*Y_cat_H2O + Gf_H*Y_cat_H; % Standard Gibbs energy of catabolic reaction
Gibbs0_an_R = Gf_NAD*Y_an_NAD + Gf_CO2*Y_an_CO2 + Gf_NH4*Y_an_NH4 + Gf_Xr*Y_an_Xr + Gf_H2O*Y_an_H2O + Gf_H*Y_an_H; % Standard Gibbs energy of anabolic reaction
Enthalpy0_cat_R = Hf_NAD*Y_cat_NAD + Hf_CO2*Y_cat_CO2 + Hf_Ac*Y_cat_Ac + Hf_H2O*Y_cat_H2O + Hf_H*Y_cat_H; % Standard Heat energy of catabolic reaction
Enthalpy0_an_R = Hf_NAD*Y_an_NAD + Hf_CO2*Y_an_CO2 + Hf_NH4*Y_an_NH4 + Hf_Xr*Y_an_Xr + Hf_H2O*Y_an_H2O + Hf_H*Y_an_H; % Standard Heat energy of anabolic reaction
% Account for Temperature
Gibbs0T_cat_R = Gibbs0_cat_R*T/T0 + Enthalpy0_cat_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of catabolic reaction
Gibbs0T_an_R = Gibbs0_an_R*T/T0 + Enthalpy0_an_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of anabolic reaction
% Account for concentrations (in mol/L)
Gibbs1T_cat_R = Gibbs0T_cat_R + R*T*(log(r_NAD/1000)*Y_cat_NAD + log(r_NADH/1000)*Y_cat_NADH + log(c_CO2/1000)*Y_cat_CO2 + log(c_Ac/1000)*Y_cat_Ac + log(c_H2O)*Y_cat_H2O + log(c_H/1000)*Y_cat_H)/1000; % Specific Gibbs energy of reaction of catabolic reaction
Gibbs1T_an_R = Gibbs0T_an_R + R*T*(log(r_NAD/1000)*Y_an_NAD + log(r_NADH/1000)*Y_an_NADH + log(c_CO2/1000)*Y_an_CO2 + log(c_NH4/1000)*Y_an_NH4 + log(c_Xr/1000)*Y_an_Xr ...
    + log(c_H2O)*Y_an_H2O + log(c_H/1000)*Y_an_H)/1000; % Specific Gibbs energy of reaction of anabolic reaction
    % Elongation Standard energies calculation
Gibbs0_But_R = Gf_NAD*2 + Gf_Ac*-2 + Gf_But*1 + Gf_H2O*2 + Gf_H*-3; % Standard Gibbs energy of butyrate elongation reaction
Gibbs0_Cap_R = Gf_NAD*2 + Gf_Ac*-1 + Gf_But*-1 + Gf_Cap*1 + Gf_H2O*2 + Gf_H*-3; % Standard Gibbs energy of caproate elongation reaction
Enthalpy0_But_R = Hf_NAD*2 + Hf_Ac*-2 + Hf_But*1 + Hf_H2O*2 + Hf_H*-3; % Standard Heat energy of butyrate elongation reaction
Enthalpy0_Cap_R = Hf_NAD*2 + Hf_Ac*-1 + Hf_But*-1 + Hf_Cap*1 + Hf_H2O*2 + Hf_H*-3; % Standard Heat energy of caproate elongation reaction
% Account for Temperature
Gibbs0T_But_R = Gibbs0_But_R*T/T0 + Enthalpy0_But_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of butyrate elongation reaction
Gibbs0T_Cap_R = Gibbs0_Cap_R*T/T0 + Enthalpy0_Cap_R*(T0-T)/T0; % Temperature specific Standard Gibbs energy of reaction of caproate elongation reaction
% Account for concentrations (in mol/L)
Gibbs1T_But_R = Gibbs0T_But_R + R*T*(log(r_NAD/1000)*2 + log(r_NADH/1000)*-2 + log(c_But/1000)*1 ...
    + log(c_Ac/1000)*-2 + log(c_H2O)*2 + log(c_H/1000)*-3)/1000; % Specific Gibbs energy of reaction of butyrate elongation reaction
Gibbs1T_Cap_R = Gibbs0T_Cap_R + R*T*(log(r_NAD/1000)*2 + log(r_NADH/1000)*-2 + log(c_Ac/1000)*-1 + log(c_But/1000)*-1 + log(c_Cap/1000)*1 ...
    + log(c_H2O)*2 + log(c_H/1000)*-3)/1000; % Specific Gibbs energy of reaction of caproate elongation reaction
% Thermodynamic product ratio (mol_i/mol_Ac)
Y_But_Ac = GR_Ac/GR_But; Y_Cap_Ac = GR_Ac/GR_Cap;
    % Calculate overall catabolic rate
lambda_cat = (Gibbs_diss_CO2 - Gibbs1T_an_R)/(Gibbs1T_cat_R - Y_But_Ac*Gibbs1T_But_R - Y_Cap_Ac*Gibbs1T_Cap_R);
    % Calculate growth stoichiometry (mol_i/mol_x)
Y_M_NADH = lambda_cat*Y_cat_NADH + Y_an_NADH; Y_M_NAD = lambda_cat*Y_cat_NAD + Y_an_NAD; Y_M_CO2 = lambda_cat*Y_cat_CO2 + Y_an_CO2; 
Y_M_NH4 = lambda_cat*Y_cat_NH4 + Y_an_NH4; Y_M_Ac = lambda_cat*Y_cat_Ac + Y_an_Ac;  
%% Specific rates - Growth, Consumption and Formation
    % Electron transfer 
k_f = k_e0*exp(-alp*2*F/R/T*(Eb - Ec)); % Forward Butler-Volmer term
k_r = k_e0*exp((1-alp)*2*F/R/T*(Eb - Ec)); % Reverse Butler-Volmer term
r_elec_m = k_f*c_NAD*c_H - k_r*c_NADH; % Electrochemical mediator rate (mol_mediator/m3/s)
    % Growth
q_CO2 = q_CO2_max*(c_CO2/(K_CO2 + c_CO2))*(c_NH4/(K_NH4 + c_NH4))*(c_NADH/(K_NADH + c_NADH))*I_Ac*I_But*I_Cap; % Specific CO2 uptake rate (mol_CO2/mol_x/h)
ms = ms_max; % Specific CO2 maintenance rate (mol_CO2/mol_x/h)
mu = (q_CO2 + ms)/Y_M_CO2; % Specific growth rate (1/h)
    % Specific Elongation production rates with Acetate & Butyrate threshold concentrations
% Acetate threshold
    A1 = 15/c_T_Ac; TC1 = 1.1*c_T_Ac; % Threshold constants
f1_Ac = 1/(1 + exp(A1*(c_T_Ac - c_Ac))); % Sigmoidal function to avoid negative rates when c_Ac < c_T_Ac
f2_Ac = 1/(1 + exp(A1*(TC1 - c_Ac))); % Sigmoidal function for smoothing the function when c_Ac is near c_T_Ac
% Butyrate threshold
    A2 = 10/c_T_But; TC2 = 1.1*c_T_But; % Threshold constants
f1_But = 1/(1 + exp(A2*(c_T_But - c_But))); % Sigmoidal function to avoid negative rates when c_But < c_T_But
f2_But = 1/(1 + exp(A2*(TC2 - c_But))); % Sigmoidal function for smoothing the function when c_But is near c_T_But
q_but = q_max_But*((c_Ac - c_T_Ac*f1_Ac)/(K_Ac + c_Ac - c_T_Ac*f1_Ac))*f2_Ac*I_Ac*I_But*I_Cap; % Specific Butyrate elongation rate (mol_But/mol_x/h)
q_Cap = q_max_Cap*((c_Ac - c_T_Ac*f1_Ac)/(K_Ac + c_Ac - c_T_Ac*f1_Ac))*f2_Ac*((c_But - c_T_But*f1_But)/(K_But + c_But - c_T_But*f1_But))*f2_But*I_Ac*I_But*I_Cap; % Specific Caproate elongation rate (mol_But/mol_x/h)
    % NAD/NADH
q_c_NADH = Y_M_NADH*mu - ms*(Y_cat_NADH/Y_cat_CO2) - 2*q_but - 2*q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_NADH/mol_x/h)
q_f_NAD = Y_M_NAD*mu - ms*(Y_cat_NAD/Y_cat_CO2) + 2*q_but + 2*q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_NAD/mol_x/h)
    % Acetate
q_t_Ac = Y_M_Ac*mu - ms*(Y_cat_Ac/Y_cat_CO2) - 2*q_but - q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_Ac/mol_x/h)
    % Butyrate
q_t_But = q_but - q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_But/mol_x/h)
    % Caproate
q_t_Cap = q_Cap; % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_Cap/mol_x/h)
    % NH4
q_t_NH4 = Y_M_NH4*mu - ms*(Y_cat_NH4/Y_cat_CO2); % Herbert-Pirt; Production rate from growth, maintenance and elongation (mol_NH4/mol_x/h)
%% Biological volumetric rates (mol_i/m3/h)
r_Xr = mu*c_Xr; % Cell rate 
r_CO2 = q_CO2*c_Xr; % CO2 rate
r_Ac = q_t_Ac*c_Xr; % Acetate rate
r_But = q_t_But*c_Xr; % Butyrate rate
r_Cap = q_t_Cap*c_Xr; % Caproate rate
r_NH4 = q_t_NH4*c_Xr; % Ammonium rate
r_bio_NADH = q_c_NADH*c_Xr; % NADH rate
r_bio_NAD = q_f_NAD*c_Xr; % NAD rate
%% Calculate derivatives
dy(1) = r_Xr; 
dy(2) = D*(0-c_CO2) + (V_fc/V_R)*r_CO2 + kla*(c_CO20-c_CO2)*(V_BC/V_R);
dy(3) = D*(0-c_Ac) + (V_fc/V_R)*r_Ac;
dy(4) = D*(0-c_But) +  (V_fc/V_R)*r_But;
dy(5) = D*(0-c_Cap) +  (V_fc/V_R)*r_Cap;
dy(6) = D*(c_NH40-c_NH4) +  (V_fc/V_R)*r_NH4;
dy(7) = (V_fc/V_R)*r_bio_NADH + (V_fc/V_R)*r_elec_m*3600;
dy(8) = (V_fc/V_R)*r_bio_NAD - (V_fc/V_R)*r_elec_m*3600;
dy=dy';
end